function pRanks = pageRank_power(M, tol, maxIter)
% Undamped PageRank calculation with the power method. If number of pages
% is <= 6, plots the probabilities as the iterations proceed. Also prints
% the amount of time taken.

% INPUTS: - M: the transition matrix
%         - tol: tolerance (for change between two consecutive iterates).
%           Default is 1e-8.
%         - maxIter: maximum number of iterations. Default is 100*size(M).

% OUTPUTS: - pRanks: PageRank vector

% Check that M is valid
if size(M,1) ~= size(M,2)
   error('Matrix M must be square') 
end
n=size(M,1);
if any(any(M)) < 0 || any(any(M)) > 1 || any(abs(1-sum(M)) > n*eps)
   error('Matrix M is not column-stochastic') 
end

% Set default parameters
if nargin < 3
   maxIter = 100*n;
   if nargin < 2
       tol = 1e-8;
   end
end

p0 = 1/n*ones(n,1); %uniform initial probability
%p0 = [1; zeros(10,1)];
done = 0; pRanks = NaN;
if n > 6 % larger number of pages -- don't plot the iterates
    p_current = p0;
    iters = 0;
    % Loop: record the current probability vector and previous probability
    % vector; terminate iterations when the norm of the difference between
    % successive iterates is less than tol
    tStart = tic;
    % Store changes between probability vectors
    P = zeros(iters);
    while ~done && iters < maxIter
        iters = iters + 1;
        p_prev = p_current;
        p_current = M*p_current;
        P(iters) = norm(p_current-p_prev);
        if P(iters) < tol %convergence
           done = 1;
           pRanks = p_current;
           P = P(:,1:iters);
        end
    end
    tEnd = toc(tStart);
    if done
        fprintf('Power method converged in %d iterations (%.5f seconds)\n',...
            iters, tEnd);
    else % no convergence
        fprintf('Power method: reached maximum iteration count (final residual norm %.4e)\n',...
            norm(p_current - p_prev));
    end
    p = semilogy(P,'LineWidth',3);
    set(gca,'fontSize',14);
    %p.LineWidth = 3;
    xlabel('Number of iterations (i)')
    ylabel('||P_i - P_{i-1}||')
else % small problems -- record iterates and plot the values
    iters = 0;
    % Record iterates in the matrix P
    P = zeros(n, maxIter+1); P(:,1) = p0;
    tStart = tic;
    while ~done && iters < maxIter
        iters = iters+1;
        P(:,iters+1) = M*P(:,iters);
        if norm(P(:,iters+1) - P(:,iters)) < tol %convergence
           done = 1;
           pRanks = P(:,iters+1);
           P = P(:,1:iters+1);
        end
    end
    tEnd = toc(tStart);
    if done
        fprintf('Power method converged in %d iterations (%.5f seconds)\n',...
            iters, tEnd);
    else % no convergence
        fprintf('Power method: reached maximum iteration count (final residual norm %.4e)\n',...
            norm(P(:,end)-P(:,end-1)));
    end
    % plotting
    figure;
    p = plot(0:iters, P(1,:)); hold on
    set(gca,'fontSize',14)
    p.LineWidth = 3;
    legendCell = {'page 1'};
    for i=2:n
       p(i) = plot(0:iters, P(i,:));
       p(i).LineWidth = 3;
       legendCell{length(legendCell)+1} = strcat('page ',num2str(i));
    end
    xlabel('Number of steps through graph')
    ylabel('Probability of being at node')
    legend(legendCell)
    maxP = max(max(P));
    axis([0 iters 0 maxP*1.5])
end

end